#!/bin/bash


vmName=$1
vmTemplate=$2
intfName=$3
vmIp=$4
vmMask=$5
vmGateway=$6


vmRootPath=/home/catapult/vm
vmImagePath=$vmRootPath/vmimages
vmImageStore=${vmName^^}

showHelp ()
{
    echo ""
    echo "    Usage:   $0  <vm_name> <vm_template> <interface_name> <vmIp> [<subnetMask> <gateway>]"
    echo ""
    echo "             vm_template can be either "".qcow2"" or "".qcow2.tar.bz2"" file format"
    echo "             For dhcp, use 'dhcp' as VM IP and no subnetMask/gateway"
    echo ""
    echo "             examples:" 
    echo "                     $0 vmone $vmRootPath/template/Ixia_Virtual_Test_Appliance_8.50_EA_KVM.qcow2.tar.gz2 eth15 dhcp"
    echo "                     $0 vmtwo $vmRootPath/template/Ixia_Virtual_Test_Appliance_9.00_KVM.qcow2 eth15 20.20.20.20 255.255.255.0 20.20.20.1"
    echo ""
    echo "List of populated/available interfaces:"
    echo ""
    $vmRootPath/utils/showSlotInfo.sh
}


cleanupExistingVm ()
{
    # Find out if the requested VM is currently defined
    local vmStatus=`sudo virsh list --all | grep $vmName`
    if [[ ! -z "$vmStatus" ]]; then
        if [[ "$vmStatus" == *"running"* ]]; then
            sudo virsh shutdown $vmName
            sleep 5
        fi
        sudo virsh undefine $vmName
        sleep 5
    fi

    sudo rm -f $vmImagePath/$vmImageStore.qcow2
}

createVmImage ()
{
    if [[ ! -f $vmTemplate ]]; then
        echo "VM template ($vmTemplate) not found"
        exit 1
    fi

    if [[ ! $vmTemplate == *.qcow2* ]]; then
        echo "VM template ($vmTemplate) no in qcow2 format"
        exit 1
    fi

    if [[ ! -d $vmImagePath ]]; then
        mkdir -p $vmImagePath
    fi

    local vmTemplateName=`echo $vmTemplate | rev | cut -d/ -f1 | rev`
    local vmTemplatePath=`echo $vmTemplate | rev | cut -d/ -f2- | rev`
 
    if [[ $vmTemplateName == *.qcow2.tar.bz2 ]]; then
        tar jxf $vmTemplate -C $vmImagePath

        vmTemplateName=${vmTemplateName::-8}
        if [[ ! -f $vmImagePath/$vmTemplateName ]]; then
            echo "VM Template file: $vmImagePath/$vmTemplateName did not extract successfully"
            exit 1
        fi
    fi
    echo "vmTemplateName now: $vmTemplateName"
    cp $vmImagePath/$vmTemplateName $vmImagePath/$vmImageStore.qcow2
}

createTheVm ()
{
    local tmpVm=/tmp/$vmName.xml
    rm -f $tmpVm

    echo "" > $tmpVm
    echo "<domain type='kvm'>" >> $tmpVm
    echo "  <name>$vmName</name>" >> $tmpVm
    echo "  <memory unit='KiB'>4194304</memory>" >> $tmpVm
    echo "  <currentMemory unit='KiB'>4194304</currentMemory>" >> $tmpVm
    echo "  <vcpu placement='static'>2</vcpu>" >> $tmpVm
    echo "  <os>" >> $tmpVm
    echo "    <type arch='x86_64' machine='pc-i440fx-bionic'>hvm</type>" >> $tmpVm
    echo "    <boot dev='hd'/>" >> $tmpVm
    echo "  </os>" >> $tmpVm
    echo "  <features>" >> $tmpVm
    echo "    <acpi/>" >> $tmpVm
    echo "    <apic/>" >> $tmpVm
    echo "    <vmport state='off'/>" >> $tmpVm
    echo "  </features>" >> $tmpVm
    echo "  <cpu mode='custom' match='exact' check='partial'>" >> $tmpVm
    echo "    <model fallback='allow'>Broadwell-IBRS</model>" >> $tmpVm
    echo "  </cpu>" >> $tmpVm
    echo "  <clock offset='utc'>" >> $tmpVm
    echo "    <timer name='rtc' tickpolicy='catchup'/>" >> $tmpVm
    echo "    <timer name='pit' tickpolicy='delay'/>" >> $tmpVm
    echo "    <timer name='hpet' present='no'/>" >> $tmpVm
    echo "  </clock>" >> $tmpVm
    echo "  <on_poweroff>destroy</on_poweroff>" >> $tmpVm
    echo "  <on_reboot>restart</on_reboot>" >> $tmpVm
    echo "  <on_crash>destroy</on_crash>" >> $tmpVm
    echo "  <pm>" >> $tmpVm
    echo "    <suspend-to-mem enabled='no'/>" >> $tmpVm
    echo "    <suspend-to-disk enabled='no'/>" >> $tmpVm
    echo "  </pm>" >> $tmpVm
    echo "  <devices>" >> $tmpVm
    echo "    <emulator>/usr/bin/kvm-spice</emulator>" >> $tmpVm
    echo "    <disk type='file' device='disk'>" >> $tmpVm
    echo "      <driver name='qemu' type='qcow2'/>" >> $tmpVm
    echo "      <source file='$vmImagePath/$vmImageStore.qcow2'/>" >> $tmpVm
    echo "      <target dev='vda' bus='virtio'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x07' function='0x0'/>" >> $tmpVm
    echo "    </disk>" >> $tmpVm
    echo "    <controller type='usb' index='0' model='ich9-ehci1'>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x7'/>" >> $tmpVm
    echo "    </controller>" >> $tmpVm
    echo "    <controller type='usb' index='0' model='ich9-uhci1'>" >> $tmpVm
    echo "      <master startport='0'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x0' multifunction='on'/>" >> $tmpVm
    echo "    </controller>" >> $tmpVm
    echo "    <controller type='usb' index='0' model='ich9-uhci2'>" >> $tmpVm
    echo "      <master startport='2'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x1'/>" >> $tmpVm
    echo "    </controller>" >> $tmpVm
    echo "    <controller type='usb' index='0' model='ich9-uhci3'>" >> $tmpVm
    echo "      <master startport='4'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x2'/>" >> $tmpVm
    echo "    </controller>" >> $tmpVm
    echo "    <controller type='pci' index='0' model='pci-root'/>" >> $tmpVm
    echo "    <controller type='virtio-serial' index='0'>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>" >> $tmpVm
    echo "    </controller>" >> $tmpVm
    echo "    <interface type='bridge'>" >> $tmpVm
    echo "      <source bridge='br0'/>" >> $tmpVm
    echo "      <model type='virtio'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>" >> $tmpVm
    echo "    </interface>" >> $tmpVm
    echo "    <interface type='direct' trustGuestRxFilters='yes'>" >> $tmpVm
    echo "      <source dev='$intfName' mode='private'/>" >> $tmpVm
    echo "      <model type='virtio'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x09' function='0x0'/>" >> $tmpVm
    echo "    </interface>" >> $tmpVm
    echo "    <serial type='pty'>" >> $tmpVm
    echo "      <target type='isa-serial' port='0'>" >> $tmpVm
    echo "        <model name='isa-serial'/>" >> $tmpVm
    echo "      </target>" >> $tmpVm
    echo "    </serial>" >> $tmpVm
    echo "    <console type='pty'>" >> $tmpVm
    echo "      <target type='serial' port='0'/>" >> $tmpVm
    echo "    </console>" >> $tmpVm
    echo "    <channel type='unix'>" >> $tmpVm
    echo "      <target type='virtio' name='org.qemu.guest_agent.0'/>" >> $tmpVm
    echo "      <address type='virtio-serial' controller='0' bus='0' port='1'/>" >> $tmpVm
    echo "    </channel>" >> $tmpVm
    echo "    <channel type='spicevmc'>" >> $tmpVm
    echo "      <target type='virtio' name='com.redhat.spice.0'/>" >> $tmpVm
    echo "      <address type='virtio-serial' controller='0' bus='0' port='2'/>" >> $tmpVm
    echo "    </channel>" >> $tmpVm
    echo "    <input type='tablet' bus='usb'>" >> $tmpVm
    echo "      <address type='usb' bus='0' port='1'/>" >> $tmpVm
    echo "    </input>" >> $tmpVm
    echo "    <input type='mouse' bus='ps2'/>" >> $tmpVm
    echo "    <input type='keyboard' bus='ps2'/>" >> $tmpVm
    echo "    <graphics type='spice' autoport='yes'>" >> $tmpVm
    echo "      <listen type='address'/>" >> $tmpVm
    echo "      <image compression='off'/>" >> $tmpVm
    echo "    </graphics>" >> $tmpVm
    echo "    <sound model='ich6'>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x04' function='0x0'/>" >> $tmpVm
    echo "    </sound>" >> $tmpVm
    echo "    <video>" >> $tmpVm
    echo "      <model type='qxl' ram='65536' vram='65536' vgamem='16384' heads='1' primary='yes'/>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x02' function='0x0'/>" >> $tmpVm
    echo "    </video>" >> $tmpVm
    echo "    <redirdev bus='usb' type='spicevmc'>" >> $tmpVm
    echo "      <address type='usb' bus='0' port='2'/>" >> $tmpVm
    echo "    </redirdev>" >> $tmpVm
    echo "    <redirdev bus='usb' type='spicevmc'>" >> $tmpVm
    echo "      <address type='usb' bus='0' port='3'/>" >> $tmpVm
    echo "    </redirdev>" >> $tmpVm
    echo "    <memballoon model='virtio'>" >> $tmpVm
    echo "      <address type='pci' domain='0x0000' bus='0x00' slot='0x08' function='0x0'/>" >> $tmpVm
    echo "    </memballoon>" >> $tmpVm
    echo "  </devices>" >> $tmpVm
    echo "</domain>" >> $tmpVm


    # Create the VM as auto start
    sudo virsh define $tmpVm
    sudo virsh autostart $vmName

    return 0
}

if [[ "$#" -ne 4 && "$#" -ne 6 ]]; then
    showHelp
    exit 1
fi

cleanupExistingVm
createVmImage

if [[ "$#" -eq 4 && "$4" != "dhcp" ]]; then
    echo "If specifying a static ip, subnetMask and gateway must be provided as well"
    exit 1
fi


if [[ $($vmRootPath/utils/showSlotInfo.sh | grep $intfName) == "" ]]; then
        echo "    Invalid interface ($intfName)"
        echo "    valid interfaces to select from:"
        $vmRootPath/utils/showSlotInfo.sh
        exit 1
fi

intfNameVM=$(sudo grep $intfName /etc/libvirt/qemu/*.xml | cut -d. -f1 | cut -d/ -f5)
if [[ ! -z "$intfNameVM" ]]; then
    if [[ "$intfNameVM" != "$vmName" ]]; then
        echo "VM $intfNameVM already has $intfName"
        exit 1
    fi
fi

# Save vm info for subsequent installs
echo "$intfName $vmIp $vmMask $vmGateway" > /home/catapult/.vmInfo/vm$vmName.info

# Create the passthrough network
$vmRootPath/utils/createPassthrough.sh $intfName

if [[ "$?" -ne 0 ]]; then
    echo "Failed to create passthrough network"
    exit 1
fi

# Create the new VM based on info provided
createTheVm

if [[ "$?" -ne 0 ]]; then
    echo "Failed to create VM $vmName"
    exit 1
fi


echo "Bringing up VM $vmName, this will take a minute"
sudo virsh start $vmName

loop=0
maxWait=60

while [[ $loop -lt $maxWait ]]; do 
    sleep 2
    printf "."
    loop=$((loop+2))
done

# Set the VM IP
$vmRootPath/utils/setVmIp.expect $vmName $vmIp $vmMask $vmGateway

if [[ "$?" == 0 ]]; then
    # Restart the VM for the new IP settings to take effect
    sudo virsh reboot $vmName
    echo "Done"
    echo ""
else
    echo "Failed to connect to the $vmName console"
    echo ""

    exit 1
fi

exit 0



